@@ -6,11 +6,6 @@ let%expect_test "[Base.Hash.state] is still immediate" =
     ignore (Sys.opaque_identity (Base.Hash.create ())));
   [%expect {| |}]
 
-let%expect_test "[Base_boot.Hash.state] is still immediate" =
-  require_no_allocation [%here] (fun () ->
-    ignore (Sys.opaque_identity (Base_boot.Hash.create ())));
-  [%expect {| |}]
-
 let%expect_test _ =
   print_s [%sexp (Caml.Obj.is_int (Caml.Obj.repr (Base.Hash.create ~seed:1 ())) : bool)];
   [%expect {| true |}];
