@@ -39,12 +39,15 @@ include (
   end
   (* Modules defined in Base *)
   with module Array := Shadow_stdlib.Array
+  with module Atomic := Shadow_stdlib.Atomic
   with module Bool := Shadow_stdlib.Bool
   with module Buffer := Shadow_stdlib.Buffer
   with module Bytes := Shadow_stdlib.Bytes
   with module Char := Shadow_stdlib.Char
+  with module Either := Shadow_stdlib.Either
   with module Float := Shadow_stdlib.Float
   with module Hashtbl := Shadow_stdlib.Hashtbl
+  with module In_channel := Shadow_stdlib.In_channel
   with module Int := Shadow_stdlib.Int
   with module Int32 := Shadow_stdlib.Int32
   with module Int64 := Shadow_stdlib.Int64
@@ -53,6 +56,7 @@ include (
   with module Map := Shadow_stdlib.Map
   with module Nativeint := Shadow_stdlib.Nativeint
   with module Option := Shadow_stdlib.Option
+  with module Out_channel := Shadow_stdlib.Out_channel
   with module Printf := Shadow_stdlib.Printf
   with module Queue := Shadow_stdlib.Queue
   with module Random := Shadow_stdlib.Random
