@@ -3,19 +3,18 @@
 
 open! Import
 
-type t = bool [@@deriving_inline enumerate, hash, sexp]
-include
-  sig
-    [@@@ocaml.warning "-32"]
-    val all : t list
-    val hash_fold_t :
-      Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-    val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-    include Ppx_sexp_conv_lib.Sexpable.S with type  t :=  t
-  end[@@ocaml.doc "@inline"]
+type t = bool [@@deriving_inline enumerate, sexp, sexp_grammar]
+
+val all : t list
+
+include Ppx_sexp_conv_lib.Sexpable.S with type t := t
+
+val t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t
+
 [@@@end]
 
 include Identifiable.S with type t := t
+include Invariant.S with type t := t
 
 (**
    - [to_int true = 1]
