@@ -3,12 +3,10 @@ include Comparable_intf
 
 module Validate (T : sig
     type t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : t -> t -> int
-        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    val compare : t -> t -> int
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
     [@@@end]
   end) : Validate with type t := T.t = struct
   module V = Validate
@@ -25,13 +23,10 @@ module Validate (T : sig
 end
 
 module With_zero (T : sig
-    type t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : t -> t -> int
-        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+    type t [@@deriving_inline compare]
+
+    val compare : t -> t -> int
+
     [@@@end]
 
     val zero : t
@@ -58,12 +53,10 @@ end
 
 module Validate_with_zero (T : sig
     type t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : t -> t -> int
-        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    val compare : t -> t -> int
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
     [@@@end]
 
     val zero : t
@@ -80,15 +73,17 @@ end
 
 module Poly (T : sig
     type t [@@deriving_inline sexp_of]
-    include
-      sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
     [@@@end]
   end) =
 struct
   module Replace_polymorphic_compare = struct
     type t = T.t [@@deriving_inline sexp_of]
+
     let sexp_of_t = (T.sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t)
+
     [@@@end]
 
     include Poly
@@ -123,8 +118,10 @@ struct
 
   include Validate (struct
       type nonrec t = t [@@deriving_inline compare, sexp_of]
+
       let compare = (compare : t -> t -> int)
       let sexp_of_t = (sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t)
+
       [@@@end]
     end)
 end
@@ -138,11 +135,41 @@ let not_equal cmp a b = cmp a b <> 0
 let min cmp t t' = if leq cmp t t' then t else t'
 let max cmp t t' = if geq cmp t t' then t else t'
 
+module Infix (T : sig
+    type t [@@deriving_inline compare]
+
+    val compare : t -> t -> int
+
+    [@@@end]
+  end) : Infix with type t := T.t = struct
+  let ( > ) a b = gt T.compare a b
+  let ( < ) a b = lt T.compare a b
+  let ( >= ) a b = geq T.compare a b
+  let ( <= ) a b = leq T.compare a b
+  let ( = ) a b = equal T.compare a b
+  let ( <> ) a b = not_equal T.compare a b
+end
+
+module Polymorphic_compare (T : sig
+    type t [@@deriving_inline compare]
+
+    val compare : t -> t -> int
+
+    [@@@end]
+  end) : Polymorphic_compare with type t := T.t = struct
+  include Infix (T)
+
+  let compare = T.compare
+  let equal = ( = )
+  let min t t' = min compare t t'
+  let max t t' = max compare t t'
+end
+
 module Make_using_comparator (T : sig
     type t [@@deriving_inline sexp_of]
-    include
-      sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
     [@@@end]
 
     include Comparator.S with type t := t
@@ -154,25 +181,8 @@ module Make_using_comparator (T : sig
   end
 
   include T
-
-  module Replace_polymorphic_compare = struct
-    module Without_squelch = struct
-      let compare = T.compare
-      let ( > ) a b = gt compare a b
-      let ( < ) a b = lt compare a b
-      let ( >= ) a b = geq compare a b
-      let ( <= ) a b = leq compare a b
-      let ( = ) a b = equal compare a b
-      let ( <> ) a b = not_equal compare a b
-      let equal = ( = )
-      let min t t' = min compare t t'
-      let max t t' = max compare t t'
-    end
-
-    include Without_squelch
-  end
-
-  include Replace_polymorphic_compare.Without_squelch
+  module Replace_polymorphic_compare = Polymorphic_compare (T)
+  include Replace_polymorphic_compare
 
   let ascending = compare
   let descending t t' = compare t' t
@@ -199,12 +209,10 @@ end
 
 module Make (T : sig
     type t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : t -> t -> int
-        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    val compare : t -> t -> int
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
     [@@@end]
   end) =
   Make_using_comparator (struct
@@ -214,21 +222,24 @@ module Make (T : sig
 
 module Inherit (C : sig
     type t [@@deriving_inline compare]
-    include sig [@@@ocaml.warning "-32"] val compare : t -> t -> int end[@@ocaml.doc
-    "@inline"]
+
+    val compare : t -> t -> int
+
     [@@@end]
   end) (T : sig
           type t [@@deriving_inline sexp_of]
-          include
-            sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-            end[@@ocaml.doc "@inline"]
+
+          val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
           [@@@end]
 
           val component : t -> C.t
         end) =
   Make (struct
     type t = T.t [@@deriving_inline sexp_of]
+
     let sexp_of_t = (T.sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t)
+
     [@@@end]
 
     let compare t t' = C.compare (T.component t) (T.component t')
