@@ -74,7 +74,7 @@ module type Comparable = sig
       {[
         module Foo = struct
           module T = struct
-            type t = ... [@@deriving_inline compare, sexp][@@@end]
+            type t = ... [@@deriving compare, sexp]
           end
           include T
           include Comparable.Make (T)
@@ -148,17 +148,37 @@ module type Comparable = sig
   val max : ('a -> 'a -> int) -> 'a -> 'a -> 'a
   val min : ('a -> 'a -> int) -> 'a -> 'a -> 'a
 
+  (** Derive [Infix] or [Polymorphic_compare] functions from just [[@@deriving compare]],
+      without need for the [sexp_of_t] required by [Make*] (see below). *)
+
+  module Infix (T : sig
+      type t [@@deriving_inline compare]
+
+      val compare : t -> t -> int
+
+      [@@@end]
+    end) : Infix with type t := T.t
+
+  module Polymorphic_compare (T : sig
+      type t [@@deriving_inline compare]
+
+      val compare : t -> t -> int
+
+      [@@@end]
+    end) : Polymorphic_compare with type t := T.t
+
   (** Inherit comparability from a component. *)
   module Inherit (C : sig
       type t [@@deriving_inline compare]
-      include sig [@@@ocaml.warning "-32"] val compare : t -> t -> int end[@@ocaml.doc
-      "@inline"]
+
+      val compare : t -> t -> int
+
       [@@@end]
     end) (T : sig
             type t [@@deriving_inline sexp_of]
-            include
-              sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-              end[@@ocaml.doc "@inline"]
+
+            val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
             [@@@end]
 
             val component : t -> C.t
@@ -166,20 +186,18 @@ module type Comparable = sig
 
   module Make (T : sig
       type t [@@deriving_inline compare, sexp_of]
-      include
-        sig
-          [@@@ocaml.warning "-32"]
-          val compare : t -> t -> int
-          val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val compare : t -> t -> int
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
     end) : S with type t := T.t
 
   module Make_using_comparator (T : sig
       type t [@@deriving_inline sexp_of]
-      include
-        sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
 
       include Comparator.S with type t := t
@@ -187,31 +205,27 @@ module type Comparable = sig
 
   module Poly (T : sig
       type t [@@deriving_inline sexp_of]
-      include
-        sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
     end) : S with type t := T.t
 
   module Validate (T : sig
       type t [@@deriving_inline compare, sexp_of]
-      include
-        sig
-          [@@@ocaml.warning "-32"]
-          val compare : t -> t -> int
-          val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val compare : t -> t -> int
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
     end) : Validate with type t := T.t
 
   module With_zero (T : sig
       type t [@@deriving_inline compare, sexp_of]
-      include
-        sig
-          [@@@ocaml.warning "-32"]
-          val compare : t -> t -> int
-          val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val compare : t -> t -> int
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
 
       val zero : t
@@ -221,12 +235,10 @@ module type Comparable = sig
 
   module Validate_with_zero (T : sig
       type t [@@deriving_inline compare, sexp_of]
-      include
-        sig
-          [@@@ocaml.warning "-32"]
-          val compare : t -> t -> int
-          val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-        end[@@ocaml.doc "@inline"]
+
+      val compare : t -> t -> int
+      val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+
       [@@@end]
 
       val zero : t
