@@ -8,14 +8,10 @@
 
 open! Import
 
-type t = float [@@deriving_inline hash]
-include
-  sig
-    [@@@ocaml.warning "-32"]
-    val hash_fold_t :
-      Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-    val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-  end[@@ocaml.doc "@inline"]
+type t = float [@@deriving_inline sexp_grammar]
+
+val t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t
+
 [@@@end]
 
 include Floatable.S with type t := t
@@ -27,6 +23,7 @@ include
   Identifiable.S with type t := t
 
 include Comparable.With_zero with type t := t
+include Invariant.S with type t := t
 
 (** [validate_ordinary] fails if class is [Nan] or [Infinite]. *)
 val validate_ordinary : t Validate.check
@@ -547,13 +544,12 @@ module Class : sig
     | Subnormal
     | Zero
   [@@deriving_inline compare, enumerate, sexp]
-  include
-    sig
-      [@@@ocaml.warning "-32"]
-      val compare : t -> t -> int
-      val all : t list
-      include Ppx_sexp_conv_lib.Sexpable.S with type  t :=  t
-    end[@@ocaml.doc "@inline"]
+
+  val compare : t -> t -> int
+  val all : t list
+
+  include Ppx_sexp_conv_lib.Sexpable.S with type t := t
+
   [@@@end]
 
   include Stringable.S with type t := t
@@ -600,11 +596,9 @@ val ieee_mantissa : t -> Int63.t
 (** S-expressions contain at most 8 significant digits. *)
 module Terse : sig
   type nonrec t = t [@@deriving_inline sexp]
-  include
-    sig
-      [@@@ocaml.warning "-32"]
-      include Ppx_sexp_conv_lib.Sexpable.S with type  t :=  t
-    end[@@ocaml.doc "@inline"]
+
+  include Ppx_sexp_conv_lib.Sexpable.S with type t := t
+
   [@@@end]
 
   include Stringable.S with type t := t
