@@ -1,7 +1,7 @@
 include Import0
 include Sexplib0.Sexp_conv
+include Sexp.Private.Raw_grammar.Builtin
 include Hash.Builtin
 include Ppx_compare_lib.Builtin
-include Int_replace_polymorphic_compare
 
 exception Not_found_s = Sexp.Not_found_s
