@@ -16,39 +16,40 @@ type 'a t =
   ; mutable elts : 'a Option_array.t
   }
 [@@deriving_inline sexp_of]
-let sexp_of_t :
-  'a . ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t =
-  fun _of_a ->
-  function
-  | { num_mutations = v_num_mutations; front = v_front; mask = v_mask;
-      length = v_length; elts = v_elts } ->
-    let bnds = [] in
-    let bnds =
-      let arg = Option_array.sexp_of_t _of_a v_elts in
-      (Ppx_sexp_conv_lib.Sexp.List
-         [Ppx_sexp_conv_lib.Sexp.Atom "elts"; arg])
-      :: bnds in
-    let bnds =
-      let arg = sexp_of_int v_length in
-      (Ppx_sexp_conv_lib.Sexp.List
-         [Ppx_sexp_conv_lib.Sexp.Atom "length"; arg])
-      :: bnds in
-    let bnds =
-      let arg = sexp_of_int v_mask in
-      (Ppx_sexp_conv_lib.Sexp.List
-         [Ppx_sexp_conv_lib.Sexp.Atom "mask"; arg])
-      :: bnds in
-    let bnds =
-      let arg = sexp_of_int v_front in
-      (Ppx_sexp_conv_lib.Sexp.List
-         [Ppx_sexp_conv_lib.Sexp.Atom "front"; arg])
-      :: bnds in
-    let bnds =
-      let arg = sexp_of_int v_num_mutations in
-      (Ppx_sexp_conv_lib.Sexp.List
-         [Ppx_sexp_conv_lib.Sexp.Atom "num_mutations"; arg])
-      :: bnds in
-    Ppx_sexp_conv_lib.Sexp.List bnds
+
+let sexp_of_t : 'a. ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t =
+  fun _of_a -> function
+    | { num_mutations = v_num_mutations
+      ; front = v_front
+      ; mask = v_mask
+      ; length = v_length
+      ; elts = v_elts
+      } ->
+      let bnds = [] in
+      let bnds =
+        let arg = Option_array.sexp_of_t _of_a v_elts in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "elts"; arg ] :: bnds
+      in
+      let bnds =
+        let arg = sexp_of_int v_length in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "length"; arg ] :: bnds
+      in
+      let bnds =
+        let arg = sexp_of_int v_mask in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "mask"; arg ] :: bnds
+      in
+      let bnds =
+        let arg = sexp_of_int v_front in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "front"; arg ] :: bnds
+      in
+      let bnds =
+        let arg = sexp_of_int v_num_mutations in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "num_mutations"; arg ]
+        :: bnds
+      in
+      Ppx_sexp_conv_lib.Sexp.List bnds
+;;
+
 [@@@end]
 
 module type S = Queue_intf.S
