@@ -4,11 +4,9 @@ open! Import
     standard [Queue] module. *)
 module type S = sig
   type 'a t [@@deriving_inline sexp]
-  include
-    sig
-      [@@@ocaml.warning "-32"]
-      include Ppx_sexp_conv_lib.Sexpable.S1 with type 'a t :=  'a t
-    end[@@ocaml.doc "@inline"]
+
+  include Ppx_sexp_conv_lib.Sexpable.S1 with type 'a t := 'a t
+
   [@@@end]
 
   include Indexed_container.S1 with type 'a t := 'a t
@@ -91,11 +89,9 @@ module type Queue = sig
   module type S = S
 
   type 'a t [@@deriving_inline compare]
-  include
-    sig
-      [@@@ocaml.warning "-32"]
-      val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
-    end[@@ocaml.doc "@inline"]
+
+  val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
+
   [@@@end]
 
   include S with type 'a t := 'a t
