@@ -11,23 +11,25 @@ module Step = struct
     | Skip of 's
     | Yield of 'a * 's
   [@@deriving_inline sexp_of]
-  let sexp_of_t : type a s.
-    (a -> Ppx_sexp_conv_lib.Sexp.t) ->
-    (s -> Ppx_sexp_conv_lib.Sexp.t) -> (a, s) t -> Ppx_sexp_conv_lib.Sexp.t
+
+  let sexp_of_t
+    : type a s.
+      (a -> Ppx_sexp_conv_lib.Sexp.t)
+      -> (s -> Ppx_sexp_conv_lib.Sexp.t)
+      -> (a, s) t
+      -> Ppx_sexp_conv_lib.Sexp.t
     =
-    fun _of_a ->
-    fun _of_s ->
-    function
-    | Done -> Ppx_sexp_conv_lib.Sexp.Atom "Done"
-    | Skip v0 ->
-      let v0 = _of_s v0 in
-      Ppx_sexp_conv_lib.Sexp.List
-        [Ppx_sexp_conv_lib.Sexp.Atom "Skip"; v0]
-    | Yield (v0, v1) ->
-      let v0 = _of_a v0
-      and v1 = _of_s v1 in
-      Ppx_sexp_conv_lib.Sexp.List
-        [Ppx_sexp_conv_lib.Sexp.Atom "Yield"; v0; v1]
+    fun _of_a _of_s -> function
+      | Done -> Ppx_sexp_conv_lib.Sexp.Atom "Done"
+      | Skip v0 ->
+        let v0 = _of_s v0 in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "Skip"; v0 ]
+      | Yield (v0, v1) ->
+        let v0 = _of_a v0
+        and v1 = _of_s v1 in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "Yield"; v0; v1 ]
+  ;;
+
   [@@@end]
 end
 
@@ -430,7 +432,7 @@ let nth s n =
 
 let nth_exn s n =
   if n < 0
-  then raise (Invalid_argument "Sequence.nth")
+  then invalid_arg "Sequence.nth"
   else (
     match nth s n with
     | None -> failwith "Sequence.nth"
@@ -443,111 +445,121 @@ module Merge_with_duplicates_element = struct
     | Right of 'b
     | Both of 'a * 'b
   [@@deriving_inline compare, hash, sexp]
+
   let compare :
-    'a 'b .
-    ('a -> 'a -> int) -> ('b -> 'b -> int) -> ('a, 'b) t -> ('a, 'b) t -> int
+    'a 'b. ('a -> 'a -> int) -> ('b -> 'b -> int) -> ('a, 'b) t -> ('a, 'b) t -> int
     =
-    fun _cmp__a ->
-    fun _cmp__b ->
-    fun a__001_ ->
-    fun b__002_ ->
+    fun _cmp__a _cmp__b a__001_ b__002_ ->
     if Ppx_compare_lib.phys_equal a__001_ b__002_
     then 0
-    else
-      (match (a__001_, b__002_) with
-       | (Left _a__003_, Left _b__004_) -> _cmp__a _a__003_ _b__004_
-       | (Left _, _) -> (-1)
-       | (_, Left _) -> 1
-       | (Right _a__005_, Right _b__006_) -> _cmp__b _a__005_ _b__006_
-       | (Right _, _) -> (-1)
-       | (_, Right _) -> 1
-       | (Both (_a__007_, _a__009_), Both (_b__008_, _b__010_)) ->
-         (match _cmp__a _a__007_ _b__008_ with
-          | 0 -> _cmp__b _a__009_ _b__010_
-          | n -> n))
-  let hash_fold_t : type a b.
-    (Ppx_hash_lib.Std.Hash.state -> a -> Ppx_hash_lib.Std.Hash.state) ->
-    (Ppx_hash_lib.Std.Hash.state -> b -> Ppx_hash_lib.Std.Hash.state) ->
-    Ppx_hash_lib.Std.Hash.state -> (a, b) t -> Ppx_hash_lib.Std.Hash.state
+    else (
+      match a__001_, b__002_ with
+      | Left _a__003_, Left _b__004_ -> _cmp__a _a__003_ _b__004_
+      | Left _, _ -> -1
+      | _, Left _ -> 1
+      | Right _a__005_, Right _b__006_ -> _cmp__b _a__005_ _b__006_
+      | Right _, _ -> -1
+      | _, Right _ -> 1
+      | Both (_a__007_, _a__009_), Both (_b__008_, _b__010_) ->
+        (match _cmp__a _a__007_ _b__008_ with
+         | 0 -> _cmp__b _a__009_ _b__010_
+         | n -> n))
+  ;;
+
+  let hash_fold_t
+    : type a b.
+      (Ppx_hash_lib.Std.Hash.state -> a -> Ppx_hash_lib.Std.Hash.state)
+      -> (Ppx_hash_lib.Std.Hash.state -> b -> Ppx_hash_lib.Std.Hash.state)
+      -> Ppx_hash_lib.Std.Hash.state
+      -> (a, b) t
+      -> Ppx_hash_lib.Std.Hash.state
     =
-    fun _hash_fold_a ->
-    fun _hash_fold_b ->
-    fun hsv ->
-    fun arg ->
-    match arg with
-    | Left _a0 ->
-      let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 0 in
-      let hsv = hsv in _hash_fold_a hsv _a0
-    | Right _a0 ->
-      let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 1 in
-      let hsv = hsv in _hash_fold_b hsv _a0
-    | Both (_a0, _a1) ->
-      let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 2 in
-      let hsv = let hsv = hsv in _hash_fold_a hsv _a0 in
-      _hash_fold_b hsv _a1
-  let t_of_sexp : type a b.
-    (Ppx_sexp_conv_lib.Sexp.t -> a) ->
-    (Ppx_sexp_conv_lib.Sexp.t -> b) -> Ppx_sexp_conv_lib.Sexp.t -> (a, b) t
+    fun _hash_fold_a _hash_fold_b hsv arg ->
+      match arg with
+      | Left _a0 ->
+        let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 0 in
+        let hsv = hsv in
+        _hash_fold_a hsv _a0
+      | Right _a0 ->
+        let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 1 in
+        let hsv = hsv in
+        _hash_fold_b hsv _a0
+      | Both (_a0, _a1) ->
+        let hsv = Ppx_hash_lib.Std.Hash.fold_int hsv 2 in
+        let hsv =
+          let hsv = hsv in
+          _hash_fold_a hsv _a0
+        in
+        _hash_fold_b hsv _a1
+  ;;
+
+  let t_of_sexp
+    : type a b.
+      (Ppx_sexp_conv_lib.Sexp.t -> a)
+      -> (Ppx_sexp_conv_lib.Sexp.t -> b)
+      -> Ppx_sexp_conv_lib.Sexp.t
+      -> (a, b) t
     =
-    let _tp_loc = "src/sequence.ml.Merge_with_duplicates_element.t" in
-    fun _of_a ->
-    fun _of_b ->
-      function
-      | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                        ("left"|"Left" as _tag))::sexp_args) as _sexp ->
+    let _tp_loc = "sequence.ml.Merge_with_duplicates_element.t" in
+    fun _of_a _of_b -> function
+      | Ppx_sexp_conv_lib.Sexp.List
+          (Ppx_sexp_conv_lib.Sexp.Atom (("left" | "Left") as _tag) :: sexp_args) as _sexp
+        ->
         (match sexp_args with
-         | v0::[] -> let v0 = _of_a v0 in Left v0
-         | _ ->
-           Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc
-             _tag _sexp)
-      | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                        ("right"|"Right" as _tag))::sexp_args) as _sexp ->
+         | [ v0 ] ->
+           let v0 = _of_a v0 in
+           Left v0
+         | _ -> Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc _tag _sexp)
+      | Ppx_sexp_conv_lib.Sexp.List
+          (Ppx_sexp_conv_lib.Sexp.Atom (("right" | "Right") as _tag) :: sexp_args) as
+        _sexp ->
         (match sexp_args with
-         | v0::[] -> let v0 = _of_b v0 in Right v0
-         | _ ->
-           Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc
-             _tag _sexp)
-      | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                        ("both"|"Both" as _tag))::sexp_args) as _sexp ->
+         | [ v0 ] ->
+           let v0 = _of_b v0 in
+           Right v0
+         | _ -> Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc _tag _sexp)
+      | Ppx_sexp_conv_lib.Sexp.List
+          (Ppx_sexp_conv_lib.Sexp.Atom (("both" | "Both") as _tag) :: sexp_args) as _sexp
+        ->
         (match sexp_args with
-         | v0::v1::[] ->
+         | [ v0; v1 ] ->
            let v0 = _of_a v0
-           and v1 = _of_b v1 in Both (v0, v1)
-         | _ ->
-           Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc
-             _tag _sexp)
-      | Ppx_sexp_conv_lib.Sexp.Atom ("left"|"Left") as sexp ->
+           and v1 = _of_b v1 in
+           Both (v0, v1)
+         | _ -> Ppx_sexp_conv_lib.Conv_error.stag_incorrect_n_args _tp_loc _tag _sexp)
+      | Ppx_sexp_conv_lib.Sexp.Atom ("left" | "Left") as sexp ->
         Ppx_sexp_conv_lib.Conv_error.stag_takes_args _tp_loc sexp
-      | Ppx_sexp_conv_lib.Sexp.Atom ("right"|"Right") as sexp ->
+      | Ppx_sexp_conv_lib.Sexp.Atom ("right" | "Right") as sexp ->
         Ppx_sexp_conv_lib.Conv_error.stag_takes_args _tp_loc sexp
-      | Ppx_sexp_conv_lib.Sexp.Atom ("both"|"Both") as sexp ->
+      | Ppx_sexp_conv_lib.Sexp.Atom ("both" | "Both") as sexp ->
         Ppx_sexp_conv_lib.Conv_error.stag_takes_args _tp_loc sexp
-      | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.List _)::_) as
-        sexp ->
+      | Ppx_sexp_conv_lib.Sexp.List (Ppx_sexp_conv_lib.Sexp.List _ :: _) as sexp ->
         Ppx_sexp_conv_lib.Conv_error.nested_list_invalid_sum _tp_loc sexp
       | Ppx_sexp_conv_lib.Sexp.List [] as sexp ->
         Ppx_sexp_conv_lib.Conv_error.empty_list_invalid_sum _tp_loc sexp
       | sexp -> Ppx_sexp_conv_lib.Conv_error.unexpected_stag _tp_loc sexp
-  let sexp_of_t : type a b.
-    (a -> Ppx_sexp_conv_lib.Sexp.t) ->
-    (b -> Ppx_sexp_conv_lib.Sexp.t) -> (a, b) t -> Ppx_sexp_conv_lib.Sexp.t
+  ;;
+
+  let sexp_of_t
+    : type a b.
+      (a -> Ppx_sexp_conv_lib.Sexp.t)
+      -> (b -> Ppx_sexp_conv_lib.Sexp.t)
+      -> (a, b) t
+      -> Ppx_sexp_conv_lib.Sexp.t
     =
-    fun _of_a ->
-    fun _of_b ->
-    function
-    | Left v0 ->
-      let v0 = _of_a v0 in
-      Ppx_sexp_conv_lib.Sexp.List
-        [Ppx_sexp_conv_lib.Sexp.Atom "Left"; v0]
-    | Right v0 ->
-      let v0 = _of_b v0 in
-      Ppx_sexp_conv_lib.Sexp.List
-        [Ppx_sexp_conv_lib.Sexp.Atom "Right"; v0]
-    | Both (v0, v1) ->
-      let v0 = _of_a v0
-      and v1 = _of_b v1 in
-      Ppx_sexp_conv_lib.Sexp.List
-        [Ppx_sexp_conv_lib.Sexp.Atom "Both"; v0; v1]
+    fun _of_a _of_b -> function
+      | Left v0 ->
+        let v0 = _of_a v0 in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "Left"; v0 ]
+      | Right v0 ->
+        let v0 = _of_b v0 in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "Right"; v0 ]
+      | Both (v0, v1) ->
+        let v0 = _of_a v0
+        and v1 = _of_b v1 in
+        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "Both"; v0; v1 ]
+  ;;
+
   [@@@end]
 end
 
@@ -663,7 +675,7 @@ let split_n s n =
 
 let chunks_exn t n =
   if n <= 0
-  then raise (Invalid_argument "Sequence.chunks_exn")
+  then invalid_arg "Sequence.chunks_exn"
   else
     unfold_step ~init:t ~f:(fun t ->
       match split_n t n with
@@ -936,7 +948,7 @@ let intersperse s ~sep =
 let repeat x = unfold_step ~init:x ~f:(fun x -> Yield (x, x))
 
 let cycle_list_exn xs =
-  if List.is_empty xs then raise (Invalid_argument "Sequence.cycle_list_exn");
+  if List.is_empty xs then invalid_arg "Sequence.cycle_list_exn";
   let s = of_list xs in
   concat_map ~f:(fun () -> s) (repeat ())
 ;;
@@ -1096,6 +1108,23 @@ let interleaved_cartesian_product s1 s2 =
   map s1 ~f:(fun x1 -> map s2 ~f:(fun x2 -> x1, x2)) |> interleave
 ;;
 
+let of_seq (seq : _ Caml.Seq.t) =
+  unfold_step ~init:seq ~f:(fun seq ->
+    match seq () with
+    | Nil -> Done
+    | Cons (hd, tl) -> Yield (hd, tl))
+;;
+
+let to_seq (Sequence (state, next)) =
+  let rec loop state =
+    match next state with
+    | Done -> Caml.Seq.Nil
+    | Skip state -> loop state
+    | Yield (hd, state) -> Caml.Seq.Cons (hd, fun () -> loop state)
+  in
+  fun () -> loop state
+;;
+
 module Generator = struct
   type 'elt steps = Wrap of ('elt, unit -> 'elt steps) Step.t
 
