@@ -5,10 +5,14 @@ type t =
   | W32
   | W64
 [@@deriving_inline sexp_of]
+
 let sexp_of_t =
   (function
     | W32 -> Ppx_sexp_conv_lib.Sexp.Atom "W32"
-    | W64 -> Ppx_sexp_conv_lib.Sexp.Atom "W64" : t -> Ppx_sexp_conv_lib.Sexp.t)
+    | W64 -> Ppx_sexp_conv_lib.Sexp.Atom "W64"
+             : t -> Ppx_sexp_conv_lib.Sexp.t)
+;;
+
 [@@@end]
 
 let num_bits = function
