@@ -4,8 +4,9 @@ open! Import
 let errors =
   [ Error.of_string "ABC"
   ; Error.tag ~tag:"DEF" (Error.of_thunk (fun () -> "GHI"))
-  ; Error.create_s ([%message "foo" ~bar:(31:int)])
+  ; Error.create_s [%message "foo" ~bar:(31 : int)]
   ]
+;;
 
 let%expect_test _ =
   List.iter errors ~f:(fun error -> show_raise (fun () -> Error.raise error));
@@ -13,6 +14,7 @@ let%expect_test _ =
     (raised ABC)
     (raised (DEF GHI))
     (raised (foo (bar 31))) |}]
+;;
 
 let%expect_test _ =
   List.iter errors ~f:(fun error ->
@@ -21,3 +23,4 @@ let%expect_test _ =
     (raised ABC)
     (raised (DEF GHI))
     (raised (foo (bar 31))) |}]
+;;
